import { LoggerLevel } from "@salesforce/core";
import { SFPowerkit } from "../../../sfpowerkit";
const BULK_THRESHOLD = 2000;
export default class BaseMetadataRetriever {
    constructor(org, tooling = false) {
        this.org = org;
        this.tooling = tooling;
        this.dataLoaded = false;
        this.cacheFileName = "";
        this.objectName = "";
    }
    setQuery(query) {
        this.query = query;
        this.countQuery = this.generateCountQuery();
    }
    generateCountQuery() {
        let queryParts = this.query.toUpperCase().split("FROM");
        let objectParts = queryParts[1].trim().split(" ");
        let objectName = objectParts[0].trim();
        this.objectName = objectName;
        let countQuery = `SELECT COUNT() FROM ${objectName}`;
        return countQuery;
    }
    async getObjects() {
        //let records: T[] = [];
        const conn = this.org.getConnection();
        if (this.tooling) {
            return executeToolingQueryAsync(this.query, conn, this.objectName);
        }
        else {
            let recordsCount = await this.getCount();
            if (recordsCount > BULK_THRESHOLD) {
                return executeBulkQueryAsync(this.query, conn, this.objectName, recordsCount);
            }
            else {
                return executeQueryAsync(this.query, conn, this.objectName);
            }
        }
    }
    async getCount() {
        SFPowerkit.log(`Count Query: ${this.countQuery}`, LoggerLevel.TRACE);
        let result = await this.org.getConnection().query(this.countQuery);
        SFPowerkit.log(`Retrieved count ${result.totalSize}`, LoggerLevel.TRACE);
        return result.totalSize;
    }
}
export async function executeToolingQueryAsync(query, conn, object) {
    let promiseQuery = new Promise((resolve, reject) => {
        let records = [];
        let hasInitProgress = false;
        let progressBar = undefined;
        if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
            SFPowerkit.logLevel === LoggerLevel.TRACE) {
            progressBar = SFPowerkit.createProgressBar(`Querying data from ${object}`, "Records fetched");
        }
        let queryRun = conn.tooling
            .query(query)
            .on("record", function (record) {
            if (!hasInitProgress) {
                hasInitProgress = true;
                if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                    SFPowerkit.logLevel === LoggerLevel.TRACE) {
                    progressBar.start(queryRun.totalSize);
                }
            }
            records.push(record);
            if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                SFPowerkit.logLevel === LoggerLevel.TRACE) {
                progressBar.increment();
            }
        })
            .on("end", function () {
            if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                SFPowerkit.logLevel === LoggerLevel.TRACE) {
                progressBar.stop();
            }
            resolve(records);
        })
            .on("error", function (error) {
            if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                SFPowerkit.logLevel === LoggerLevel.TRACE) {
                progressBar.stop();
            }
            reject(error);
        })
            .run({
            autoFetch: true,
            maxFetch: 1000000
        });
    });
    return promiseQuery;
}
export async function executeBulkQueryAsync(query, conn, object, recordCount) {
    let promiseQuery = new Promise((resolve, reject) => {
        let records = [];
        let hasInitProgress = false;
        let progressBar = undefined;
        SFPowerkit.log(`Using Bulk API`, LoggerLevel.DEBUG);
        if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
            SFPowerkit.logLevel === LoggerLevel.TRACE) {
            progressBar = SFPowerkit.createProgressBar(`Querying data from ${object}`, "Records fetched");
        }
        conn.bulk
            .query(query)
            .on("record", function (record) {
            if (!hasInitProgress) {
                hasInitProgress = true;
                if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                    SFPowerkit.logLevel === LoggerLevel.TRACE) {
                    progressBar.start(recordCount);
                }
            }
            records.push(record);
            if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                SFPowerkit.logLevel === LoggerLevel.TRACE) {
                progressBar.increment();
            }
        })
            .on("end", function () {
            if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                SFPowerkit.logLevel === LoggerLevel.TRACE) {
                progressBar.stop();
            }
            resolve(records);
        })
            .on("error", function (error) {
            if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                SFPowerkit.logLevel === LoggerLevel.TRACE) {
                progressBar.stop();
            }
            SFPowerkit.log(`Error when using bulk api `, LoggerLevel.ERROR);
            SFPowerkit.log(error, LoggerLevel.ERROR);
            reject(error);
        });
    });
    return promiseQuery;
}
export async function executeQueryAsync(query, conn, object) {
    let promiseQuery = new Promise((resolve, reject) => {
        let records = [];
        let hasInitProgress = false;
        let progressBar = undefined;
        if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
            SFPowerkit.logLevel === LoggerLevel.TRACE) {
            progressBar = SFPowerkit.createProgressBar(`Querying data from ${object}`, "Records fetched");
        }
        let queryRun = conn
            .query(query)
            .on("record", function (record) {
            if (!hasInitProgress) {
                hasInitProgress = true;
                if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                    SFPowerkit.logLevel === LoggerLevel.TRACE) {
                    progressBar.start(queryRun.totalSize);
                }
            }
            records.push(record);
            if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                SFPowerkit.logLevel === LoggerLevel.TRACE) {
                progressBar.increment();
            }
        })
            .on("end", function () {
            if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                SFPowerkit.logLevel === LoggerLevel.TRACE) {
                progressBar.stop();
            }
            resolve(records);
        })
            .on("error", function (error) {
            if (SFPowerkit.logLevel === LoggerLevel.DEBUG ||
                SFPowerkit.logLevel === LoggerLevel.TRACE) {
                progressBar.stop();
            }
            reject(error);
        })
            .run({
            autoFetch: true,
            maxFetch: 1000000
        });
    });
    return promiseQuery;
}
//# sourceMappingURL=baseMetadataRetriever.js.map